/*
 *                      Yeppp! library build framework
 *
 * This file is part of Yeppp! library infrastructure and licensed under 2-clause BSD license.
 * See library/LICENSE.txt for details.
 *
 */

package info.yeppp.Build;

public enum ABI {

	X86_Linux_Pic_i586,
	X86_Linux_Pic_Android,
	X86_Windows_Default_i586,
	X86_NaCl_NaCl_i686,

	X64_Linux_SystemV_Default,
	X64_Windows_Microsoft_Default,
	X64_NaCl_NaCl_Default,

	ARM_Linux_OABI_V4T,
	ARM_Linux_SoftEABI_V5T,
	ARM_Linux_HardEABI_V7A,
	ARM_Linux_SoftEABI_Android,
	ARM_Linux_SoftEABI_AndroidV7A,
	ARM_NaCl_NaCl_Default,

	MIPS_Linux_O32_Android,

	IA64_Windows_Default_Default;

	public static enum LowLevelABI {
		X86_Pic,
		X86_NonPic,
		X86_NaCl,
		X64_Microsoft,
		X64_SystemV,
		X64_NaCl,
		ARM_OABI,
		ARM_SoftEABI,
		ARM_HardEABI,
		ARM_NaCl,
		MIPS_O32,
		IA64;

		@Override
		public String toString() {
			switch (this) {
				case X86_Pic:
					return "x86-pic";
				case X86_NonPic:
					return "x86-nonpic";
				case X86_NaCl:
					return "x86-nacl";
				case X64_Microsoft:
					return "x64-ms";
				case X64_SystemV:
					return "x64-sysv";
				case X64_NaCl:
					return "x64-nacl";
				case ARM_OABI:
					return "arm-oabi";
				case ARM_SoftEABI:
					return "arm-softeabi";
				case ARM_HardEABI:
					return "arm-hardeabi";
				case ARM_NaCl:
					return "arm-nacl";
				case MIPS_O32:
					return "mips-o32";
				case IA64:
					return "ia64-default";
				default:
					return super.toString();
			}
		}


	}

	@Override
	public String toString() {
		switch (this) {
			case X86_Linux_Pic_i586:
				return "x86-linux-pic-i586";
			case X86_Linux_Pic_Android:
				return "x86-linux-pic-android";
			case X86_Windows_Default_i586:
				return "x86-windows-nopic-i586";
			case X86_NaCl_NaCl_i686:
				return "x86-nacl-pic-i686";
			case X64_Linux_SystemV_Default:
				return "x64-linux-sysv-default";
			case X64_Windows_Microsoft_Default:
				return "x64-windows-ms-default";
			case X64_NaCl_NaCl_Default:
				return "x64-nacl-sysv-default";
			case ARM_Linux_OABI_V4T:
				return "arm-linux-oabi-v4t";
			case ARM_Linux_SoftEABI_V5T:
				return "arm-linux-softeabi-v5t";
			case ARM_Linux_HardEABI_V7A:
				return "arm-linux-hardeabi-v7a";
			case ARM_Linux_SoftEABI_Android:
				return "arm-linux-softeabi-android";
			case ARM_Linux_SoftEABI_AndroidV7A:
				return "arm-linux-softeabi-androidv7a";
			case ARM_NaCl_NaCl_Default:
				return "arm-nacl-nacl-default";
			case MIPS_Linux_O32_Android:
				return "mips-linux-o32-android";
			case IA64_Windows_Default_Default:
				return "ia64-windows-default-default";
			default:
				return super.toString();
		}
	}

	public LowLevelABI getLowLevelABI() {
		switch (this) {
			case X86_Linux_Pic_i586:
				return LowLevelABI.X86_Pic;
			case X86_Linux_Pic_Android:
				return LowLevelABI.X86_Pic;
			case X86_Windows_Default_i586:
				return LowLevelABI.X86_NonPic;
			case X86_NaCl_NaCl_i686:
				return LowLevelABI.X86_NaCl;
			case X64_Linux_SystemV_Default:
				return LowLevelABI.X64_SystemV;
			case X64_Windows_Microsoft_Default:
				return LowLevelABI.X64_Microsoft;
			case X64_NaCl_NaCl_Default:
				return LowLevelABI.X64_NaCl;
			case ARM_Linux_OABI_V4T:
				return LowLevelABI.ARM_OABI;
			case ARM_Linux_SoftEABI_V5T:
				return LowLevelABI.ARM_SoftEABI;
			case ARM_Linux_HardEABI_V7A:
				return LowLevelABI.ARM_HardEABI;
			case ARM_Linux_SoftEABI_Android:
				return LowLevelABI.ARM_SoftEABI;
			case ARM_Linux_SoftEABI_AndroidV7A:
				return LowLevelABI.ARM_SoftEABI;
			case ARM_NaCl_NaCl_Default:
				return LowLevelABI.ARM_NaCl;
			case MIPS_Linux_O32_Android:
				return LowLevelABI.MIPS_O32;
			case IA64_Windows_Default_Default:
				return LowLevelABI.IA64;
			default:
				return null;
		}
	}

	public OperatingSystem getOS() {
		switch (this) {
			case X86_Linux_Pic_i586:
				return OperatingSystem.Linux;
			case X86_Linux_Pic_Android:
				return OperatingSystem.Linux;
			case X86_Windows_Default_i586:
				return OperatingSystem.Windows;
			case X86_NaCl_NaCl_i686:
				return OperatingSystem.NativeClient;
			case X64_Linux_SystemV_Default:
				return OperatingSystem.Linux;
			case X64_Windows_Microsoft_Default:
				return OperatingSystem.Windows;
			case X64_NaCl_NaCl_Default:
				return OperatingSystem.NativeClient;
			case ARM_Linux_OABI_V4T:
				return OperatingSystem.Linux;
			case ARM_Linux_SoftEABI_V5T:
				return OperatingSystem.Linux;
			case ARM_Linux_HardEABI_V7A:
				return OperatingSystem.Linux;
			case ARM_Linux_SoftEABI_Android:
				return OperatingSystem.Linux;
			case ARM_Linux_SoftEABI_AndroidV7A:
				return OperatingSystem.Linux;
			case ARM_NaCl_NaCl_Default:
				return OperatingSystem.NativeClient;
			case MIPS_Linux_O32_Android:
				return OperatingSystem.Linux;
			case IA64_Windows_Default_Default:
				return OperatingSystem.Windows;
			default:
				return null;
		}
	}

	public Architecture getArchitecture() {
		switch (this) {
			case X86_Linux_Pic_i586:
				return Architecture.X86;
			case X86_Linux_Pic_Android:
				return Architecture.X86;
			case X86_Windows_Default_i586:
				return Architecture.X86;
			case X86_NaCl_NaCl_i686:
				return Architecture.X86;
			case X64_Linux_SystemV_Default:
				return Architecture.X64;
			case X64_Windows_Microsoft_Default:
				return Architecture.X64;
			case X64_NaCl_NaCl_Default:
				return Architecture.X64;
			case ARM_Linux_OABI_V4T:
				return Architecture.ARM;
			case ARM_Linux_SoftEABI_V5T:
				return Architecture.ARM;
			case ARM_Linux_HardEABI_V7A:
				return Architecture.ARM;
			case ARM_Linux_SoftEABI_Android:
				return Architecture.ARM;
			case ARM_Linux_SoftEABI_AndroidV7A:
				return Architecture.ARM;
			case ARM_NaCl_NaCl_Default:
				return Architecture.ARM;
			case MIPS_Linux_O32_Android:
				return Architecture.MIPS;
			case IA64_Windows_Default_Default:
				return Architecture.IA64;
			default:
				return null;
		}
	}

}