/*
 *                      Yeppp! library build framework
 *
 * This file is part of Yeppp! library infrastructure and licensed under 2-clause BSD license.
 * See library/LICENSE.txt for details.
 *
 */

package info.yeppp.Build;

import info.yeppp.Build.internals.FileSystem;
import org.apache.commons.io.FilenameUtils;
import org.apache.commons.lang3.SystemUtils;

import java.util.*;

public abstract class AbstractJavaCompiler implements JavaCompiler {
	public String getObjectPath(String sourcePath) {
		if (sourceDirectory == null) {
			throw new IllegalStateException("Source directory was not assigned any value");
		}
		if (classDirectory == null) {
			throw new IllegalStateException("Object directory was not assigned any value");
		}
		final String relativeSourcePath = FileSystem.getRelativePath(sourcePath, sourceDirectory);
		if (relativeSourcePath == null) {
			throw new IllegalArgumentException("Source path is not within source directory");
		}
		return FilenameUtils.concat(classDirectory,
				FilenameUtils.removeExtension(relativeSourcePath) + ".class");
	}

	public void setSourceDirectory(String sourceDirectory) {
		this.sourceDirectory = sourceDirectory;
	}

	public String getSourceDirectory() {
		return this.sourceDirectory;
	}

	public void setClassDirectory(String classDirectory) {
		this.classDirectory = classDirectory;
	}

	public String getClassDirectory() {
		return this.classDirectory;
	}

	public void setDebugEnabled(boolean debugEnabled) {
		this.debugEnabled = debugEnabled;
	}

	public boolean getDebugEnabled() {
		return this.debugEnabled;
	}

	public boolean addClassPathDirectory(String classPath) {
		return classPathDirectories.add(FilenameUtils.normalizeNoEndSeparator(classPath));
	}

	public boolean removeClassPathDirectory(String classPath) {
		return classPathDirectories.remove(FilenameUtils.normalizeNoEndSeparator(classPath));
	}

	public void clearClassPathDirectories() {
		classPathDirectories.clear();
	}

	protected String compilerPath = null;
	protected String sourceDirectory = null;
	protected String classDirectory = null;
	protected Set<String> classPathDirectories = new HashSet<String>();
	protected boolean debugEnabled = false;
}
