/*
 *                      Yeppp! library build framework
 *
 * This file is part of Yeppp! library infrastructure and licensed under 2-clause BSD license.
 * See library/LICENSE.txt for details.
 *
 */

package info.yeppp.Build;

import org.apache.commons.io.FilenameUtils;

import java.util.ArrayList;
import java.util.List;

public abstract class AbstractLibrarian implements Librarian {

	public void setBinariesDirectory(String binariesDirectory) {
		this.binariesDirectory = binariesDirectory;
	}

	public String getBinariesDirectory() {
		return this.binariesDirectory;
	}

	public void setObjectDirectory(String objectDirectory) {
		this.objectDirectory = objectDirectory;
	}

	public String getObjectDirectory() {
		return this.objectDirectory;
	}

	public boolean addLibraryDependence(String libraryName) {
		return libraries.add(libraryName);
	}

	public boolean removeLibraryDependence(String libraryName) {
		return libraries.remove(libraryName);
	}

	public void clearLibraryDependences() {
		libraries.clear();
	}

	public boolean addGlobalLibraryDirectory(String libraryDirectory) {
		return globalLibraryDirectories.add(FilenameUtils.normalizeNoEndSeparator(libraryDirectory));
	}

	public boolean removeGlobalLibraryDirectory(String libraryDirectory) {
		return globalLibraryDirectories.remove(FilenameUtils.normalizeNoEndSeparator(libraryDirectory));
	}

	public void clearGlobalLibraryDirectories() {
		globalLibraryDirectories.clear();
	}

	public boolean addLibraryDirectory(String libraryDirectory) {
		return libraryDirectories.add(FilenameUtils.normalizeNoEndSeparator(libraryDirectory));
	}

	public boolean removeLibraryDirectory(String libraryDirectory) {
		return libraryDirectories.remove(FilenameUtils.normalizeNoEndSeparator(libraryDirectory));
	}

	public void clearLibraryDirectories() {
		libraryDirectories.clear();
	}

	protected String binariesDirectory = null;
	protected String objectDirectory = null;
	protected List<String> libraries = new ArrayList<String>();
	protected List<String> globalLibraryDirectories = new ArrayList<String>();
	protected List<String> libraryDirectories = new ArrayList<String>();

}
