/*
 *                      Yeppp! library build framework
 *
 * This file is part of Yeppp! library infrastructure and licensed under 2-clause BSD license.
 * See library/LICENSE.txt for details.
 *
 */

package info.yeppp.Build;

import info.yeppp.Build.internals.WinRegistry;
import info.yeppp.Build.internals.FileSystem;
import info.yeppp.Build.sdk.WindowsSdk;

public class VisualStudio2010 extends VisualStudio {
	public VisualStudio2010(WindowsSdk windowsSdk) {
		super(windowsSdk);
		try {
			installPath = WinRegistry.readString(WinRegistry.HKEY_LOCAL_MACHINE, "Software\\Microsoft\\VisualStudio\\10.0", "InstallDir");
		} catch (Exception e) {
		}
		if (installPath == null) {
			try {
				installPath = WinRegistry.readString(WinRegistry.HKEY_LOCAL_MACHINE, "Software\\WOW6432Node\\Microsoft\\VisualStudio\\10.0", "InstallDir");
			} catch (Exception e) {
			}
		}
		if (installPath == null) {
			throw new UnsupportedOperationException("Visual Studio 2010 was not found on this system and can not be used");
		}
		installPath = FileSystem.lookupExistingDirectory(installPath, "..\\..\\");
		if (installPath == null) {
			throw new UnsupportedOperationException("Visual Studio 2010 install directory was not found on disk");
		}
		idePath = FileSystem.lookupExistingDirectory(installPath, "Common7\\IDE\\");
		if (idePath == null) {
			throw new UnsupportedOperationException("Visual Studio 2010 IDE directory was not found on disk");
		}
		commonToolsPath = FileSystem.lookupExistingDirectory(installPath, "Common7\\Tools\\");
		if (commonToolsPath == null) {
			throw new UnsupportedOperationException("Visual Studio 2010 common tools directory was not found on disk");
		}

		cppIncludePath = FileSystem.lookupExistingDirectory(installPath, "VC\\include");

		cppToolsPathX86 = lookupCppToolchain("VC\\bin\\", "ml");
		cppLibraryPathX86 = FileSystem.lookupExistingDirectory(installPath, "VC\\lib\\");

		cppToolsPathX64 = lookupCppToolchain("VC\\bin\\amd64", "ml64");
		if (cppToolsPathX64 == null) {
			cppToolsPathX64 = lookupCppToolchain("VC\\bin\\x86_amd64", "ml64");
			isCrossChainX64 = cppToolsPathX64 != null;
		}
		cppLibraryPathX64 = FileSystem.lookupExistingDirectory(installPath, "VC\\lib\\amd64\\");

		cppToolsPathIA64 = lookupCppToolchain("VC\\bin\\ia64", "ias");
		if (cppToolsPathIA64 == null) {
			cppToolsPathIA64 = lookupCppToolchain("VC\\bin\\x86_ia64", "ias");
			isCrossChainIA64 = cppToolsPathIA64 != null;
		}
		cppLibraryPathIA64 = FileSystem.lookupExistingDirectory(installPath, "VC\\lib\\ia64");
	}

	public String getName() {
		return "Visual Studio 2010";
	}
	
	public String getShortName() {
		return "vs10";
	}

}
