/*
 *                      Yeppp! library build framework
 *
 * This file is part of Yeppp! library infrastructure and licensed under 2-clause BSD license.
 * See library/LICENSE.txt for details.
 *
 */

package info.yeppp.Build.gui;

import javax.swing.*;
import javax.swing.border.Border;
import javax.swing.border.EmptyBorder;
import java.awt.*;

class BuildListCellRenderer extends JLabel implements ListCellRenderer {
	private static final Border NO_FOCUS_BORDER = new EmptyBorder(1, 1, 1, 1);

	private BorderedFilledColorIcon icon;

	public BuildListCellRenderer() {
		icon = new BorderedFilledColorIcon(16, 4, Color.GRAY, null);
		setIcon(icon);
		setOpaque(true);
	}

	public Component getListCellRendererComponent(JList list, Object value,
	                                              int index, boolean isSelected, boolean hasFocus) {
		// The object from the combo box model MUST be a ColorName.
		BuildItem buildItem = (BuildItem) value;

		// Sets icon and text on the JLabel.
		icon.setFillColor(buildItem.getColor());
		setText(buildItem.getName());

		if (isSelected) {
			setBackground(list.getSelectionBackground());
			setForeground(list.getSelectionForeground());
		} else {
			setBackground(list.getBackground());
			setForeground(list.getForeground());
		}

		if (hasFocus) {
			setBorder(UIManager.getBorder("List.focusCellHighlightBorder"));
		} else {
			setBorder(NO_FOCUS_BORDER);
		}

		setComponentOrientation(list.getComponentOrientation());
		setEnabled(list.isEnabled());
		setFont(list.getFont());

		return this;
	}

	// The following methods are overridden to be empty for performance
	// reasons. If you want to understand better why, please read:
	//
	// http://java.sun.com/javase/6/docs/api/javax/swing/DefaultListCellRenderer.html#override

	public void validate() {}
	public void invalidate() {}
	public void repaint() {}
	public void revalidate() {}
	public void repaint(long tm, int x, int y, int width, int height) {}
	public void repaint(Rectangle r) {}
}
                      

