/*
 *                      Yeppp! library build framework
 *
 * This file is part of Yeppp! library infrastructure and licensed under 2-clause BSD license.
 * See library/LICENSE.txt for details.
 *
 */

/*
 * Author:  andbin - http://andbin.altervista.org
 * License: Public Domain
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 */

package info.yeppp.Build.gui;

import info.yeppp.Build.*;
import info.yeppp.Build.sdk.Java6Sdk;
import info.yeppp.Build.sdk.Windows7Sdk;
import info.yeppp.Build.sdk.Windows8Sdk;
import info.yeppp.Build.sdk.WindowsSdk;
import org.apache.commons.lang3.SystemUtils;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

class BuildVisualizationFrame extends JFrame {
	private JList buildList;
	private JScrollPane scrollPane;
	private JButton buildButton;
	private JPanel buildPanel;
	private JComboBox toolchainSelect;
	private JTextArea messageArea;
	private JScrollPane messagePane;

	public BuildVisualizationFrame() {
		super("Yeppp Build Framework");

		setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
		setSize(640, 480);

		// Creates a DefaultListModel that contains ColorName objects.
		final DefaultListModel listModel = new DefaultListModel();
		listModel.addElement(new BuildItem("Library"));
		listModel.addElement(new BuildItem("Performance Tests"));
		listModel.addElement(new BuildItem("Examples"));
		listModel.addElement(new BuildItem("JNI Classes"));
		listModel.addElement(new BuildItem("JNI Library"));

		buildList = new JList(listModel);
		buildList.setCellRenderer(new BuildListCellRenderer());

		scrollPane = new JScrollPane(buildList);

		messageArea = new JTextArea();
		messageArea.setBackground(Color.WHITE);
		messageArea.setForeground(Color.BLACK);
		messageArea.setEditable(false);
		messagePane = new JScrollPane(messageArea);
		messagePane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);
		messagePane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);

		getContentPane().add(scrollPane, BorderLayout.CENTER);

		DefaultComboBoxModel toolChainDataModel = new DefaultComboBoxModel();
		try {
			final Architecture architecture = Architecture.X64;
			final LibraryBuilder.Toolchain toolchain = new LibraryBuilder.Toolchain(architecture, "x64-vs2012");
			final WindowsSdk windowsSdk = new Windows8Sdk();
			final VisualStudio2011 visualStudio = new VisualStudio2011(windowsSdk);
			final Java6Sdk javaSdk = new Java6Sdk();
			toolchain.compiler = new MicrosoftCppCompiler(visualStudio, windowsSdk, architecture);
			toolchain.javaCompiler = javaSdk.new JavaCompiler();
			toolchain.javaSdk = javaSdk;
			toolchain.linker = new MicrosoftLinker(visualStudio, windowsSdk, architecture);
			toolchain.librarian = visualStudio.new Librarian(architecture);
			toolchain.resourceCompiler = new ResourceCompiler(windowsSdk, visualStudio);
			toolchain.nasm = new NetwideAssembler();
			toolChainDataModel.addElement(toolchain);
		} catch (Exception e) {
		}
		try {
			final Architecture architecture = Architecture.X64;
			final LibraryBuilder.Toolchain toolchain = new LibraryBuilder.Toolchain(architecture, "x64-vs2010");
			final WindowsSdk windowsSdk = new Windows7Sdk();
			final VisualStudio visualStudio = new VisualStudio2010(windowsSdk);
			final Java6Sdk javaSdk = new Java6Sdk();
			toolchain.compiler = new MicrosoftCppCompiler(visualStudio, windowsSdk, architecture);
			toolchain.javaCompiler = javaSdk.new JavaCompiler();
			toolchain.javaSdk = javaSdk;
			toolchain.linker = new MicrosoftLinker(visualStudio, windowsSdk, architecture);
			toolchain.librarian = visualStudio.new Librarian(architecture);
			toolchain.resourceCompiler = new ResourceCompiler(windowsSdk, visualStudio);
			toolchain.nasm = new NetwideAssembler();
			toolChainDataModel.addElement(toolchain);
		} catch (Exception e) {
		}
		try {
			final Architecture architecture = Architecture.X64;
			final LibraryBuilder.Toolchain toolchain = new LibraryBuilder.Toolchain(architecture, "x64-vs2008");
			final WindowsSdk windowsSdk = new Windows7Sdk();
			final VisualStudio visualStudio = new VisualStudio2008(windowsSdk);
			toolchain.compiler = new MicrosoftCppCompiler(visualStudio, windowsSdk, architecture);
			toolchain.linker = new MicrosoftLinker(visualStudio, windowsSdk, architecture);
			toolchain.librarian = visualStudio.new Librarian(architecture);
			toolchain.resourceCompiler = new ResourceCompiler(windowsSdk, visualStudio);
			toolchain.nasm = new NetwideAssembler();
			toolChainDataModel.addElement(toolchain);
		} catch (Exception e) {
		}
		try {
			final Architecture architecture = Architecture.X86;
			final LibraryBuilder.Toolchain toolchain = new LibraryBuilder.Toolchain(architecture, "x86-vs2012");
			final WindowsSdk windowsSdk = new Windows8Sdk();
			final VisualStudio visualStudio = new VisualStudio2011(windowsSdk);
			toolchain.compiler = new MicrosoftCppCompiler(visualStudio, windowsSdk, architecture);
			toolchain.linker = new MicrosoftLinker(visualStudio, windowsSdk, architecture);
			toolchain.librarian = visualStudio.new Librarian(architecture);
			toolchain.resourceCompiler = new ResourceCompiler(windowsSdk, visualStudio);
			toolchain.nasm = new NetwideAssembler();
			toolChainDataModel.addElement(toolchain);
		} catch (Exception e) {
		}
		try {
			final Architecture architecture = Architecture.X86;
			final LibraryBuilder.Toolchain toolchain = new LibraryBuilder.Toolchain(architecture, "x86-vs2010");
			final WindowsSdk windowsSdk = new Windows7Sdk();
			final VisualStudio visualStudio = new VisualStudio2010(windowsSdk);
			final Java6Sdk javaSdk = new Java6Sdk();
			toolchain.compiler = new MicrosoftCppCompiler(visualStudio, windowsSdk, architecture);
			toolchain.javaCompiler = javaSdk.new JavaCompiler();
			toolchain.javaSdk = javaSdk;
			toolchain.linker = new MicrosoftLinker(visualStudio, windowsSdk, architecture);
			toolchain.librarian = visualStudio.new Librarian(architecture);
			toolchain.resourceCompiler = new ResourceCompiler(windowsSdk, visualStudio);
			toolchain.nasm = new NetwideAssembler();
			toolChainDataModel.addElement(toolchain);
		} catch (Exception e) {
		}
		try {
			final Architecture architecture = Architecture.X86;
			final LibraryBuilder.Toolchain toolchain = new LibraryBuilder.Toolchain(architecture, "x86-vs2008");
			final WindowsSdk windowsSdk = new Windows7Sdk();
			final VisualStudio visualStudio = new VisualStudio2008(windowsSdk);
			toolchain.compiler = new MicrosoftCppCompiler(visualStudio, windowsSdk, architecture);
			toolchain.linker = new MicrosoftLinker(visualStudio, windowsSdk, architecture);
			toolchain.librarian = visualStudio.new Librarian(architecture);
			toolchain.resourceCompiler = new ResourceCompiler(windowsSdk, visualStudio);
			toolchain.nasm = new NetwideAssembler();
			toolChainDataModel.addElement(toolchain);
		} catch (Exception e) {
		}
		try {
			final Architecture architecture = Architecture.ARM;
			final LibraryBuilder.Toolchain toolchain = new LibraryBuilder.Toolchain(architecture, "arm-vs2012");
			final WindowsSdk windowsSdk = new Windows8Sdk();
			final VisualStudio visualStudio = new VisualStudio2011(windowsSdk);
			toolchain.compiler = new MicrosoftCppCompiler(visualStudio, windowsSdk, architecture);
			toolchain.linker = new MicrosoftLinker(visualStudio, windowsSdk, architecture);
			toolchain.librarian = visualStudio.new Librarian(architecture);
			toolchain.resourceCompiler = new ResourceCompiler(windowsSdk, visualStudio);
			toolchain.nasm = new NetwideAssembler();
			toolChainDataModel.addElement(toolchain);
		} catch (Exception e) {
		}
		try {
			final Architecture architecture = Architecture.IA64;
			final LibraryBuilder.Toolchain toolchain = new LibraryBuilder.Toolchain(architecture, "ia64-vs2010");
			final WindowsSdk windowsSdk = new Windows7Sdk();
			final VisualStudio visualStudio = new VisualStudio2010(windowsSdk);
			toolchain.compiler = new MicrosoftCppCompiler(visualStudio, windowsSdk, architecture);
			toolchain.linker = new MicrosoftLinker(visualStudio, windowsSdk, architecture);
			toolchain.librarian = visualStudio.new Librarian(architecture);
			toolchain.resourceCompiler = new ResourceCompiler(windowsSdk, visualStudio);
			toolchain.nasm = new NetwideAssembler();
			toolChainDataModel.addElement(toolchain);
		} catch (Exception e) {
		}
		buildPanel = new JPanel();

		toolchainSelect = new JComboBox(toolChainDataModel);
		buildPanel.add(toolchainSelect, BorderLayout.CENTER);

		buildButton = new JButton("Build!");
		buildButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				buildButton.setEnabled(false);
				messageArea.setText("");
				messageArea.paintImmediately(messageArea.getBounds());

				Thread buildThread = new Thread() {
					@Override
					public void run() {
						for (int i = 0; i < listModel.size(); i++) {
							BuildItem buildItem = (BuildItem) listModel.get(i);
							buildItem.setState(BuildItem.State.Waiting);
						}
						buildList.paintImmediately(buildList.getCellBounds(0, listModel.size() - 1));

						LibraryBuilder.Toolchain toolchain = (LibraryBuilder.Toolchain) toolchainSelect.getSelectedItem();
						final String projectDirectory = System.getProperty("user.dir");
						LibraryBuilder builder = new LibraryBuilder(projectDirectory, toolchain);
						try {
							((BuildItem) listModel.get(0)).setState(BuildItem.State.Building);
							buildList.paintImmediately(buildList.getCellBounds(0, 0));
							BuildMessages buildMessages = builder.compileLibrary();
							if (buildMessages.hasErrors()) {
								StringBuilder stringBuilder = new StringBuilder();
								for (BuildMessage buildMessage : buildMessages.iterable()) {
									stringBuilder.append(buildMessage.toString() + SystemUtils.LINE_SEPARATOR);
								}
								messageArea.setText(stringBuilder.toString());
								throw new Exception();
							}
							((BuildItem) listModel.get(0)).setState(BuildItem.State.Built);
							((BuildItem) listModel.get(1)).setState(BuildItem.State.Building);
							buildList.repaint();
//							buildList.paintImmediately(buildList.getCellBounds(0, 1));
							buildMessages = builder.buildPerformanceTests();
							if (buildMessages.hasErrors()) {
								StringBuilder stringBuilder = new StringBuilder();
								for (BuildMessage buildMessage : buildMessages.iterable()) {
									stringBuilder.append(buildMessage.toString() + SystemUtils.LINE_SEPARATOR);
								}
								messageArea.setText(stringBuilder.toString());
								throw new Exception();
							}
							((BuildItem) listModel.get(1)).setState(BuildItem.State.Built);
							((BuildItem) listModel.get(2)).setState(BuildItem.State.Building);
							buildList.repaint();
//							buildList.paintImmediately(buildList.getCellBounds(1, 2));
							buildMessages = builder.buildExamples();
							if (buildMessages.hasErrors()) {
								StringBuilder stringBuilder = new StringBuilder();
								for (BuildMessage buildMessage : buildMessages.iterable()) {
									stringBuilder.append(buildMessage.toString() + SystemUtils.LINE_SEPARATOR);
								}
								messageArea.setText(stringBuilder.toString());
								throw new Exception();
							}
							((BuildItem) listModel.get(2)).setState(BuildItem.State.Built);
							((BuildItem) listModel.get(3)).setState(BuildItem.State.Building);
							buildList.repaint();
//							buildList.paintImmediately(buildList.getCellBounds(2, 3));
							buildMessages = builder.buildJniClasses();
							if (buildMessages.hasErrors()) {
								StringBuilder stringBuilder = new StringBuilder();
								for (BuildMessage buildMessage : buildMessages.iterable()) {
									stringBuilder.append(buildMessage.toString() + SystemUtils.LINE_SEPARATOR);
								}
								messageArea.setText(stringBuilder.toString());
								throw new Exception();
							}
							((BuildItem) listModel.get(3)).setState(BuildItem.State.Built);
							((BuildItem) listModel.get(4)).setState(BuildItem.State.Building);
							buildList.repaint();
//							buildList.paintImmediately(buildList.getCellBounds(3, 4));
							buildMessages = builder.buildJniLibrary();
							if (buildMessages.hasErrors()) {
								StringBuilder stringBuilder = new StringBuilder();
								for (BuildMessage buildMessage : buildMessages.iterable()) {
									stringBuilder.append(buildMessage.toString() + SystemUtils.LINE_SEPARATOR);
								}
								messageArea.setText(stringBuilder.toString());
								throw new Exception();
							}
							((BuildItem) listModel.get(4)).setState(BuildItem.State.Built);
							buildList.repaint();
//							buildList.paintImmediately(buildList.getCellBounds(4, 4));
						} catch (Exception exception) {
							for (int i = 0; i < listModel.size(); i++) {
								BuildItem buildItem = (BuildItem) listModel.get(i);
								BuildItem.State state = buildItem.getState();
								if (!state.equals(BuildItem.State.Built)) {
									buildItem.setState(BuildItem.State.Failed);
									buildList.repaint();
//									buildList.paint(buildList.getCellBounds(i, i));
								}
							}
						}
						buildButton.setEnabled(true);
					}
				};
				buildThread.start();
			}
		});
		buildPanel.add(buildButton, BorderLayout.EAST);
		getContentPane().add(buildPanel, BorderLayout.NORTH);
		getContentPane().add(messagePane, BorderLayout.SOUTH);
		pack();
		messagePane.setPreferredSize(new Dimension(messagePane.getWidth(), 150));
	}
}


