/*
 *                      Yeppp! library build framework
 *
 * This file is part of Yeppp! library infrastructure and licensed under 2-clause BSD license.
 * See library/LICENSE.txt for details.
 *
 */

package info.yeppp.Build.sdk;

import info.yeppp.Build.internals.FileSystem;
import info.yeppp.Build.internals.WinRegistry;

public class Windows7Sdk extends WindowsSdk {
	public Windows7Sdk() {
		try {
			installPath = WinRegistry.readString(WinRegistry.HKEY_LOCAL_MACHINE, "Software\\Microsoft\\Microsoft SDKs\\Windows\\v7.0A", "InstallationFolder");
		} catch (Exception e) {
		}
		if (installPath == null) {
			throw new UnsupportedOperationException("Windows SDK 7 was not found on this system and can not be used");
		}
		installPath = FileSystem.lookupExistingDirectory(installPath, ".\\");
		if (installPath == null) {
			throw new UnsupportedOperationException("Windows SDK 7 install directory was not found on disk");
		}
		includePath = FileSystem.lookupExistingDirectory(installPath, "Include\\");
		if (includePath == null) {
			throw new UnsupportedOperationException("Windows SDK 7 include directory was not found on disk");
		}

		libraryX86Path = FileSystem.lookupExistingDirectory(installPath, "Lib\\");
		libraryX64Path = FileSystem.lookupExistingDirectory(installPath, "Lib\\x64\\");
		libraryIA64Path = FileSystem.lookupExistingDirectory(installPath, "Lib\\IA64\\");
	}
	
	public String getName() {
		return "Windows 7 SDK";
	}
}
