/*
 *                      Yeppp! library build framework
 *
 * This file is part of Yeppp! library infrastructure and licensed under 2-clause BSD license.
 * See library/LICENSE.txt for details.
 *
 */

package info.yeppp.Build.sdk;

import info.yeppp.Build.internals.FileSystem;
import info.yeppp.Build.internals.WinRegistry;

public class Windows8Sdk extends WindowsSdk {
	public Windows8Sdk() {
		try {
			installPath = WinRegistry.readString(WinRegistry.HKEY_LOCAL_MACHINE, "Software\\Wow6432Node\\Microsoft\\Microsoft SDKs\\Windows\\v8.0", "InstallationFolder");
		} catch (Exception e) {
		}
		if (installPath == null) {
			throw new UnsupportedOperationException("Windows SDK 8 was not found on this system and can not be used");
		}
		installPath = FileSystem.lookupExistingDirectory(installPath, ".\\");
		if (installPath == null) {
			throw new UnsupportedOperationException("Windows SDK 8 install directory was not found on disk");
		}
		includePath = FileSystem.lookupExistingDirectory(installPath, "Include\\um\\");
		if (includePath == null) {
			throw new UnsupportedOperationException("Windows SDK 8 include directory was not found on disk");
		}
		sharedIncludePath = FileSystem.lookupExistingDirectory(installPath, "Include\\shared\\");
		if (sharedIncludePath == null) {
			throw new UnsupportedOperationException("Windows SDK 8 shared include directory was not found on disk");
		}

		libraryX86Path = FileSystem.lookupExistingDirectory(installPath, "Lib\\win8\\um\\x86\\");
		libraryX64Path = FileSystem.lookupExistingDirectory(installPath, "Lib\\win8\\um\\x64\\");
		libraryArmPath = FileSystem.lookupExistingDirectory(installPath, "Lib\\win8\\um\\arm\\");
	}

	public String getSharedIncludePath() {
		return sharedIncludePath;
	}
	
	public String getName() {
		return "Windows 8 SDK";
	}

	private String sharedIncludePath = null;
}
