#
#       Portable Efficient Assembly Code-generator in Higher-level Python (PEACH-Py)
#
# This file is part of Yeppp! library infrastructure and licensed under 2-clause BSD license.
# See codegen/LICENSE.txt for details.
#

__author__ = 'Marat'

from yeppp.codegen.x64 import *

def Log_V64f_V64f_implementation(codegen, function_signature, module, function, inputs, outputs, arguments):
	if codegen.abi.name in ['x64-ms', 'x64-sysv']:
		if module == 'Math':
			if function in ['Log']:
				x_argument = arguments[0]
				y_argument = arguments[1]
				length_argument = arguments[2]

				x_type = x_argument.get_type().get_primitive_type()
				y_type = y_argument.get_type().get_primitive_type()

				if not x_type.is_floating_point() or not y_type.is_floating_point():
					return

				x_size = x_type.get_size(codegen.abi)
				y_size = y_type.get_size(codegen.abi)

				if x_size != 8 or y_size != 8:
					return

				parameters = [x_argument, y_argument, length_argument]
				codegen.begin_function(function_signature, parameters, 'Nehalem')
				if codegen.abi.name == 'x64-ms':
					xPointer = rcx
					yPointer = rdx
					length = r8
				else:
					xPointer = rdi
					yPointer = rsi
					length = rdx

				c2  = Constant.float64x2('-0x1.0000000000004p-1')
				c3  = Constant.float64x2( '0x1.5555555555524p-2')
				c4  = Constant.float64x2('-0x1.FFFFFFFFFD99Cp-3')
				c5  = Constant.float64x2( '0x1.99999999A037Bp-3')
				c6  = Constant.float64x2('-0x1.5555555707EC0p-3')
				c7  = Constant.float64x2( '0x1.24924922E0177p-3')
				c8  = Constant.float64x2('-0x1.FFFFFEE6E9A74p-4')
				c9  = Constant.float64x2( '0x1.C71C7190489D4p-4')
				c10 = Constant.float64x2('-0x1.9999CAC62E0ADp-4')
				c11 = Constant.float64x2( '0x1.745D4E8088327p-4')
				c12 = Constant.float64x2('-0x1.55505B48AD061p-4')
				c13 = Constant.float64x2( '0x1.3B0A137482D51p-4')
				c14 = Constant.float64x2('-0x1.24DCC4EF07DB3p-4')
				c15 = Constant.float64x2( '0x1.11DFFAEDD082Ep-4')
				c16 = Constant.float64x2('-0x1.FB780993249D2p-5')
				c17 = Constant.float64x2( '0x1.CF5EFF4973CFAp-5')
				c18 = Constant.float64x2('-0x1.D0CACFA711D8Cp-5')
				c19 = Constant.float64x2( '0x1.0BA1125E4CABDp-4')
				c20 = Constant.float64x2('-0x1.ED49EDACFD1C0p-5')
				c21 = Constant.float64x2( '0x1.AE7E3413896B4p-6')
				ln2_hi = Constant.float64x2('0x1.62E42FEFA3800p-1')
				ln2_lo = Constant.float64x2('0x1.EF35793C76730p-45')
				magic_mask = Constant.float64x2('0x1.0000000000040p+52')
				magic_bias = Constant.float64x2('0x1.000000000043Fp+52')
				sqrt2 = Constant.float64x2('0x1.6A09E667F3BCDp+0')
				one = Constant.float64x2('0x1.0000000000000p+0')
				mantissa_mask = Constant.uint64x2(0x000FFFFFFFFFFFFFL)
				half_exp_mask = Constant.uint64x2(0x0010000000000000L)

				sign_mask = Constant.uint64x2(0x8000000000000000L)
				min_normal = Constant.float64x2('0x1.0000000000000p-1022')
				denormal_exp_shift = Constant.uint64x2(0x0400000000000000L)
				special_magic_mask = Constant.float64x2('0x1.0000000000000p+52')
				nan = Constant.uint64x2(0x7FF8000000000000L)
				plus_inf = Constant.uint64x2(0x7FF0000000000000)
				minus_inf = Constant.uint64x2(0xFFF0000000000000)

				e0 = LocalVariable(SSERegister)
				e1 = LocalVariable(SSERegister)
				e2 = LocalVariable(SSERegister)
				e3 = LocalVariable(SSERegister)
				e4 = LocalVariable(SSERegister)
				e5 = LocalVariable(SSERegister)
				e6 = LocalVariable(SSERegister)
				e7 = LocalVariable(SSERegister)
				e8 = LocalVariable(SSERegister)
				e9 = LocalVariable(SSERegister)

				LOAD.PARAMETER( xPointer, x_argument )
				LOAD.PARAMETER( yPointer, y_argument )
				LOAD.PARAMETER( length, length_argument )

				def scalar_log(xPointer, yPointer):
					xmm_denormal_exp_shift = xmm12
					xmm_one = xmm13
					xmm_exp = xmm10
					xmm_min_normal = xmm14
					xmm_special_magic_mask = xmm15

					LOAD.CONSTANT( xmm_denormal_exp_shift, denormal_exp_shift )
					LOAD.CONSTANT( xmm_one, one )
					LOAD.CONSTANT( xmm_min_normal, min_normal )
					LOAD.CONSTANT( xmm_special_magic_mask, special_magic_mask )

					def FULL_RANGE_REDUCTION(xmm_temp, exp_var):
						MOVSD( xmm_exp, [xPointer] )
						MOVAPS( xmm11, xmm_exp )
						CMPLTSD( xmm_exp, xmm_min_normal )
						MOVDQA( xmm_temp, xmm_one )
						PAND( xmm_exp, xmm_denormal_exp_shift )
						PADDD( xmm_temp, xmm_exp )
						PXOR( xmm_exp, xmm_denormal_exp_shift )
						MULSD( xmm_temp, xmm11 )
						PADDD( xmm_exp, xmm_temp )
						PSRLQ( xmm_exp, 52 )

						ANDPS( xmm_temp, mantissa_mask )
						ORPS( xmm_temp, xmm_one )

						LOAD.CONSTANT( xmm11, sqrt2 )
						CMPLESD( xmm11, xmm_temp)
						PSUBQ( xmm_exp, xmm11 )
						POR( xmm_exp, xmm_special_magic_mask )
						PAND( xmm11, half_exp_mask )
						PSUBQ( xmm_temp, xmm11 )
						MOVDQA( exp_var, xmm_exp )

					FULL_RANGE_REDUCTION( xmm0, e0 )

					LOAD.CONSTANT( xmm12, nan )
					LOAD.CONSTANT( xmm13, minus_inf )
					PCMPEQB( xmm11, xmm11 )

					def MAP_NEGATIVE(xmm_temp):
						MOVSD( xmm10, [xPointer] )
						PCMPGTQ( xmm10, xmm11 )
						PAND( xmm_temp, xmm10 )
						PANDN( xmm10, xmm12 )
						POR( xmm_temp, xmm10 )

					MAP_NEGATIVE( xmm0 )

					PXOR( xmm11, xmm11 )

					def MAP_ZERO(xmm_temp):
						MOVSD( xmm10, [xPointer] )
						PCMPEQQ( xmm10, xmm11 )
						PAND( xmm10, xmm13 )
						POR( xmm_temp, xmm10 )

					MAP_ZERO( xmm0 )

					LOAD.CONSTANT( xmm11, plus_inf )

					def MAP_INF(xmm_temp):
						MOVSD( xmm10, [xPointer] )
						PCMPEQQ( xmm10, xmm11 )
						PAND( xmm10, xmm11 )
						POR( xmm_temp, xmm10 )

					MAP_INF( xmm0 )

					def MAP_NAN(xmm_temp):
						MOVSD( xmm10, [xPointer] )
						MOVAPS( xmm11, xmm10 )
						CMPEQSD( xmm10, xmm10 )
						PAND( xmm_temp, xmm10 )
						PANDN( xmm10, xmm11 )
						POR( xmm_temp, xmm10 )

					MAP_NAN( xmm0 )

					LOAD.CONSTANT( xmm_one, one )
					SUBPD( xmm0, xmm_one )

					LOAD.CONSTANT( xmm4, c21 )

					def HORNER_STEP(constant):
						LOAD.CONSTANT( xmm15, constant )
						MULSD( xmm4, xmm0 )
						ADDSD( xmm4, xmm15 )

					HORNER_STEP(c20)
					HORNER_STEP(c19)
					HORNER_STEP(c18)
					HORNER_STEP(c17)
					HORNER_STEP(c16)
					HORNER_STEP(c15)
					HORNER_STEP(c14)
					HORNER_STEP(c13)
					HORNER_STEP(c12)
					HORNER_STEP(c11)
					HORNER_STEP(c10)
					HORNER_STEP(c9)
					HORNER_STEP(c8)
					HORNER_STEP(c7)
					HORNER_STEP(c6)
					HORNER_STEP(c5)
					HORNER_STEP(c4)
					HORNER_STEP(c3)
					HORNER_STEP(c2)

					LOAD.CONSTANT( xmm15, magic_bias )

					MOVAPS( xmm14, e0 )
					MULSD( xmm4, xmm0 )
					SUBSD( xmm14, xmm15 )
					MULSD( xmm4, xmm0 )
					ADDSD( xmm4, xmm0 )
					MOVAPS( e0, xmm14 )

					LOAD.CONSTANT( xmm2, ln2_hi )

					MOVAPS( xmm0, e0 )
					MOVAPS( xmm1, xmm3 )
					MULPD( xmm1, xmm0 )
					ADDPD( xmm4, xmm1 )
					MULPD( xmm0, xmm2 )
					ADDPD( xmm4, xmm0 )
					MOVSD( [yPointer], xmm4 )

				TEST( xPointer, 15 )
				JZ( 'xA16' )
				LABEL( 'xM16')
				scalar_log(xPointer, yPointer)
				ADD( xPointer, 8 )
				ADD( yPointer, 8 )
				SUB( length, 1 )
				JZ( 'finish' )

				TEST( xPointer, 15 )
				JNZ( 'xM16' )

				LABEL( 'xA16' )

				SUB( length, 20 )
				
				JB( 'restore' )
				ALIGN(32)
				LABEL( 'process_by_20' )
				LOAD.CONSTANT( xmm12, magic_mask )
				LOAD.CONSTANT( xmm13, half_exp_mask )
				LOAD.CONSTANT( xmm14, mantissa_mask )
				LOAD.CONSTANT( xmm15, one )
				xmm_one = xmm15

				def FAST_RANGE_REDUCTION(xmm_temp, exp_var, index):
					MOVDQA( xmm_temp, [xPointer + index * 16] )
					MOV( eax, [xPointer + index * 16 + 4] )
					MOVDQA( xmm10, xmm_temp )
					PAND( xmm_temp, xmm14 )
					SUB( eax, 0x80100000 )
					PSRLQ( xmm10, 52 )
					POR( xmm_temp, xmm15 )
					CMP( eax, 0xFFD00000 )
					JG( 'special' )
					LOAD.CONSTANT( xmm11, sqrt2 )
					CMPLEPD( xmm11, xmm_temp )
					MOV( eax, [xPointer + index * 16 + 12] )
					PSUBQ( xmm10, xmm11 )
					PADDQ( xmm10, xmm12 )
					SUB( eax, 0x80100000 )
					PAND( xmm11, xmm13 )
					PSUBQ( xmm_temp, xmm11 )
					CMP( eax, 0xFFD00000 )
					JG( 'special' )
					MOVDQA( exp_var, xmm10 )
				
				FAST_RANGE_REDUCTION(xmm0, e0, 0)
				FAST_RANGE_REDUCTION(xmm1, e1, 1)
				FAST_RANGE_REDUCTION(xmm2, e2, 2)
				FAST_RANGE_REDUCTION(xmm3, e3, 3)
				FAST_RANGE_REDUCTION(xmm4, e4, 4)
				FAST_RANGE_REDUCTION(xmm5, e5, 5)
				FAST_RANGE_REDUCTION(xmm6, e6, 6)
				FAST_RANGE_REDUCTION(xmm7, e7, 7)
				FAST_RANGE_REDUCTION(xmm8, e8, 8)
				FAST_RANGE_REDUCTION(xmm9, e9, 9)

				LABEL( 'horner' )

				x0 = xmm0
				x1 = xmm1
				x2 = xmm2
				x3 = xmm3
				x4 = LocalVariable(SSERegister)
				x5 = LocalVariable(SSERegister)
				x6 = LocalVariable(SSERegister)
				x7 = LocalVariable(SSERegister)
				x8 = LocalVariable(SSERegister)
				x9 = LocalVariable(SSERegister)
			
				SUBPD( xmm0, xmm_one )
				SUBPD( xmm1, xmm_one )
				SUBPD( xmm2, xmm_one )
				SUBPD( xmm3, xmm_one )
				SUBPD( xmm4, xmm_one )
				MOVAPS( x4, xmm4 )
				SUBPD( xmm5, xmm_one )
				MOVAPS( x5, xmm5 )
				SUBPD( xmm6, xmm_one )
				MOVAPS( x6, xmm6 )
				SUBPD( xmm7, xmm_one )
				MOVAPS( x7, xmm7 )
				SUBPD( xmm8, xmm_one )
				MOVAPS( x8, xmm8 )
				SUBPD( xmm9, xmm_one )
				MOVAPS( x9, xmm9 )

				LOAD.CONSTANT( xmm12, c21 )
				MOVAPS( xmm13, xmm12 )
				LOAD.CONSTANT( xmm11, c21 )
				MOVAPS( xmm10, xmm11 )
				LOAD.CONSTANT( xmm4, c21 )
				MOVAPS( xmm5, xmm4 )
				MOVAPS( xmm6, xmm4 )
				MOVAPS( xmm7, xmm4 )
				MOVAPS( xmm8, xmm4 )
				MOVAPS( xmm9, xmm4 )

				def HORNER_STEP(constant):
					LOAD.CONSTANT( xmm15, constant )
					MULPD( xmm4, xmm0 )
					ADDPD( xmm4, xmm15 )
					MULPD( xmm5, xmm1 )
					ADDPD( xmm5, xmm15 )
					MULPD( xmm6, xmm2 )
					ADDPD( xmm6, xmm15 )
					MULPD( xmm7, xmm3 )
					ADDPD( xmm7, xmm15 )
					MOVAPS( xmm14, x4 )
					MULPD( xmm8, xmm14 )
					ADDPD( xmm8, xmm15 )
					MOVAPS( xmm14, x5 )
					MULPD( xmm9, xmm14 )
					ADDPD( xmm9, xmm15 )
					MOVAPS( xmm14, x6 )
					MULPD( xmm10, xmm14 )
					ADDPD( xmm10, xmm15 )
					MOVAPS( xmm14, x7 )
					MULPD( xmm11, xmm14 )
					ADDPD( xmm11, xmm15 )
					MOVAPS( xmm14, x8 )
					MULPD( xmm12, xmm14 )
					ADDPD( xmm12, xmm15 )
					MOVAPS( xmm14, x9 )
					MULPD( xmm13, xmm14 )
					ADDPD( xmm13, xmm15 )

				HORNER_STEP(c20)
				HORNER_STEP(c19)
				HORNER_STEP(c18)
				HORNER_STEP(c17)
				HORNER_STEP(c16)
				HORNER_STEP(c15)
				HORNER_STEP(c14)
				HORNER_STEP(c13)
				HORNER_STEP(c12)
				HORNER_STEP(c11)
				HORNER_STEP(c10)
				HORNER_STEP(c9)
				HORNER_STEP(c8)
				HORNER_STEP(c7)
				HORNER_STEP(c6)
				HORNER_STEP(c5)
				HORNER_STEP(c4)
				HORNER_STEP(c3)
				HORNER_STEP(c2)


				LOAD.CONSTANT( xmm15, magic_bias )
				
				MOVAPS( xmm14, e0 )
				MULPD( xmm4, xmm0 )
				SUBPD( xmm14, xmm15 )
				MULPD( xmm4, xmm0 )
				ADDPD( xmm4, xmm0 )
				MOVAPS( e0, xmm14 )

				MOVAPS( xmm0, e1 )
				MULPD( xmm5, xmm1 )
				SUBPD( xmm0, xmm15 )
				MULPD( xmm5, xmm1 )
				ADDPD( xmm5, xmm1 )
				MOVAPS( e1, xmm0 )

				MOVAPS( xmm1, e2 )
				MULPD( xmm6, xmm2 )
				SUBPD( xmm1, xmm15 )
				MULPD( xmm6, xmm2 )
				ADDPD( xmm6, xmm2 )
				MOVAPS( e2, xmm1 )

				MOVAPS( xmm1, e3 )
				MULPD( xmm7, xmm3 )
				SUBPD( xmm1, xmm15 )
				MOVAPS( xmm0, x4 )
				MULPD( xmm7, xmm3 )
				ADDPD( xmm7, xmm3 )
				MOVAPS( e3, xmm1 )

				MOVAPS( xmm2, e4 )
				MULPD( xmm8, xmm0 )
				SUBPD( xmm2, xmm15 )
				MOVAPS( xmm1, x5 )
				MULPD( xmm8, xmm0 )
				ADDPD( xmm8, xmm0 )
				MOVAPS( e4, xmm2 )

				MOVAPS( xmm2, e5 )
				MULPD( xmm9, xmm1 )
				SUBPD( xmm2, xmm15 )
				MOVAPS( xmm0, x6 )
				MULPD( xmm9, xmm1 )
				ADDPD( xmm9, xmm1 )
				MOVAPS( e5, xmm2 )

				MOVAPS( xmm2, e6 )
				MULPD( xmm10, xmm0 )
				SUBPD( xmm2, xmm15 )
				MOVAPS( xmm1, x7 )
				MULPD( xmm10, xmm0 )
				ADDPD( xmm10, xmm0 )
				MOVAPS( e6, xmm2 )

				MOVAPS( xmm2, e7 )
				MULPD( xmm11, xmm1 )
				SUBPD( xmm2, xmm15 )
				MOVAPS( xmm0, x8 )
				MULPD( xmm11, xmm1 )
				ADDPD( xmm11, xmm1 )
				MOVAPS( e7, xmm2 )

				MOVAPS( xmm2, e8 )
				MULPD( xmm12, xmm0 )
				SUBPD( xmm2, xmm15 )
				MOVAPS( xmm1, x9 )
				MULPD( xmm12, xmm0 )
				ADDPD( xmm12, xmm0 )
				LOAD.CONSTANT( xmm3, ln2_lo )
				MOVAPS( e8, xmm2 )

				MOVAPS( xmm2, e9 )
				MULPD( xmm13, xmm1 )
				SUBPD( xmm2, xmm15 )
				MOVAPS( xmm0, e0 )
				MULPD( xmm13, xmm1 )
				ADDPD( xmm13, xmm1 )
				MOVAPS( e9, xmm2 )

				LOAD.CONSTANT( xmm2, ln2_hi )

				MOVAPS( xmm1, xmm3 )
				MULPD( xmm1, xmm0 )
				ADDPD( xmm4, xmm1 )
				MULPD( xmm0, xmm2 )
				ADDPD( xmm4, xmm0 )
				MOVAPS( xmm0, e1 )
				MOVUPS( [yPointer], xmm4 )

				MOVAPS( xmm1, xmm3 )
				MULPD( xmm1, xmm0 )
				ADDPD( xmm5, xmm1 )
				MOVAPS( xmm4, e2 )
				MULPD( xmm0, xmm2 )
				ADDPD( xmm5, xmm0 )
				MOVUPS( [yPointer + 16], xmm5 )

				MOVAPS( xmm1, xmm3 )
				MULPD( xmm1, xmm4 )
				ADDPD( xmm6, xmm1 )
				MOVAPS( xmm5, e3 )
				MULPD( xmm4, xmm2 )
				ADDPD( xmm6, xmm4 )
				MOVUPS( [yPointer + 32], xmm6 )

				MOVAPS( xmm1, xmm3 )
				MULPD( xmm1, xmm5 )
				ADDPD( xmm7, xmm1 )
				MOVAPS( xmm6, e4 )
				MULPD( xmm5, xmm2 )
				ADDPD( xmm7, xmm5 )
				MOVUPS( [yPointer + 48], xmm7 )

				MOVAPS( xmm1, xmm3 )
				MULPD( xmm1, xmm6 )
				ADDPD( xmm8, xmm1 )
				MOVAPS( xmm7, e5 )
				MULPD( xmm6, xmm2 )
				ADDPD( xmm8, xmm6 )
				MOVUPS( [yPointer + 64], xmm8 )

				MOVAPS( xmm1, xmm3 )
				MULPD( xmm1, xmm7 )
				ADDPD( xmm9, xmm1 )
				MOVAPS( xmm8, e6 )
				MULPD( xmm7, xmm2 )
				ADDPD( xmm9, xmm7 )
				MOVUPS( [yPointer + 80], xmm9 )

				MOVAPS( xmm1, xmm3 )
				MULPD( xmm1, xmm8 )
				ADDPD( xmm10, xmm1 )
				MOVAPS( xmm9, e7 )
				MULPD( xmm8, xmm2 )
				ADDPD( xmm10, xmm8 )
				MOVUPS( [yPointer + 96], xmm10 )

				MOVAPS( xmm1, xmm3 )
				MULPD( xmm1, xmm9 )
				ADDPD( xmm11, xmm1 )
				MOVAPS( xmm10, e8 )
				MULPD( xmm9, xmm2 )
				ADDPD( xmm11, xmm9 )
				MOVUPS( [yPointer + 112], xmm11 )

				MOVAPS( xmm1, xmm3 )
				MULPD( xmm1, xmm10 )
				ADDPD( xmm12, xmm1 )
				MOVAPS( xmm11, e9 )
				MULPD( xmm10, xmm2 )
				ADDPD( xmm12, xmm10 )
				MOVUPS( [yPointer + 128], xmm12 )

				MOVAPS( xmm1, xmm3 )
				MULPD( xmm1, xmm11 )
				ADDPD( xmm13, xmm1 )
				MULPD( xmm11, xmm2 )
				ADDPD( xmm13, xmm11 )
				MOVUPS( [yPointer + 144], xmm13 )

				ADD( xPointer, 160 )
				ADD( yPointer, 160 )
				SUB( length, 20 )
				JAE( 'process_by_20' )

				LABEL( 'restore' )
				ADD( length, 20 )
				JZ( 'finish' )

				LABEL( 'process_by_1' )
				scalar_log(xPointer, yPointer)
				ADD( xPointer, 8 )
				ADD( yPointer, 8 )
				SUB( length, 1 )
				JNZ( 'process_by_1' )
				
				LABEL( 'finish' )
				RETURN( 0 )

				LABEL( 'special' )
			
				LOAD.CONSTANT( xmm12, denormal_exp_shift )
				LOAD.CONSTANT( xmm13, one )
				LOAD.CONSTANT( xmm14, min_normal )
				LOAD.CONSTANT( xmm15, special_magic_mask )

				def FULL_RANGE_REDUCTION(xmm_temp, exp_var, index):
					MOVAPS( xmm10, [xPointer + index * 16] )
					MOVAPS( xmm11, xmm10 )
					CMPLTPD( xmm10, xmm14 )
					MOVDQA( xmm_temp, xmm13 )
					PAND( xmm10, xmm12 )
					PADDD( xmm_temp, xmm10 )
					PXOR( xmm10, xmm12 )
					MULPD( xmm_temp, xmm11 )
					PADDD( xmm10, xmm_temp )
					PSRLQ( xmm10, 52 )

					ANDPS( xmm_temp, mantissa_mask )
					ORPS( xmm_temp, xmm13 )

					LOAD.CONSTANT( xmm11, sqrt2 )
					CMPLEPD( xmm11, xmm_temp)
					PSUBQ( xmm10, xmm11 )
					POR( xmm10, xmm15 )
					PAND( xmm11, half_exp_mask )
					PSUBQ( xmm_temp, xmm11 )
					MOVDQA( exp_var, xmm10 )

				FULL_RANGE_REDUCTION( xmm0, e0, 0 )
				FULL_RANGE_REDUCTION( xmm1, e1, 1 )
				FULL_RANGE_REDUCTION( xmm2, e2, 2 )
				FULL_RANGE_REDUCTION( xmm3, e3, 3 )
				FULL_RANGE_REDUCTION( xmm4, e4, 4 )
				FULL_RANGE_REDUCTION( xmm5, e5, 5 )
				FULL_RANGE_REDUCTION( xmm6, e6, 6 )
				FULL_RANGE_REDUCTION( xmm7, e7, 7 )
				FULL_RANGE_REDUCTION( xmm8, e8, 8 )
				FULL_RANGE_REDUCTION( xmm9, e9, 9 )

				LOAD.CONSTANT( xmm12, nan )
				LOAD.CONSTANT( xmm13, minus_inf )
				PCMPEQB( xmm11, xmm11 )

				def MAP_NEGATIVE(xmm_temp, index):
					MOVDQA( xmm10, [xPointer + index * 16] )
					PCMPGTQ( xmm10, xmm11 )
					PAND( xmm_temp, xmm10 )
					PANDN( xmm10, xmm12 )
					POR( xmm_temp, xmm10 )

				MAP_NEGATIVE( xmm0, 0 )
				MAP_NEGATIVE( xmm1, 1 )
				MAP_NEGATIVE( xmm2, 2 )
				MAP_NEGATIVE( xmm3, 3 )
				MAP_NEGATIVE( xmm4, 4 )
				MAP_NEGATIVE( xmm5, 5 )
				MAP_NEGATIVE( xmm6, 6 )
				MAP_NEGATIVE( xmm7, 7 )
				MAP_NEGATIVE( xmm8, 8 )
				MAP_NEGATIVE( xmm9, 9 )

				PXOR( xmm11, xmm11 )				

				def MAP_ZERO(xmm_temp, index):
					MOVDQA( xmm10, [xPointer + index * 16] )
					PCMPEQQ( xmm10, xmm11 )
					PAND( xmm10, xmm13 )
					POR( xmm_temp, xmm10 )

				MAP_ZERO( xmm0, 0 )
				MAP_ZERO( xmm1, 1 )
				MAP_ZERO( xmm2, 2 )
				MAP_ZERO( xmm3, 3 )
				MAP_ZERO( xmm4, 4 )
				MAP_ZERO( xmm5, 5 )
				MAP_ZERO( xmm6, 6 )
				MAP_ZERO( xmm7, 7 )
				MAP_ZERO( xmm8, 8 )
				MAP_ZERO( xmm9, 9 )


				LOAD.CONSTANT( xmm11, plus_inf )

				def MAP_INF(xmm_temp, index):
					MOVDQA( xmm10, [xPointer + index * 16] )
					PCMPEQQ( xmm10, xmm11 )
					PAND( xmm10, xmm11 )
					POR( xmm_temp, xmm10 )

				MAP_INF( xmm0, 0 )
				MAP_INF( xmm1, 1 )
				MAP_INF( xmm2, 2 )
				MAP_INF( xmm3, 3 )
				MAP_INF( xmm4, 4 )
				MAP_INF( xmm5, 5 )
				MAP_INF( xmm6, 6 )
				MAP_INF( xmm7, 7 )
				MAP_INF( xmm8, 8 )
				MAP_INF( xmm9, 9 )

				def MAP_NAN(xmm_temp, index):
					MOVAPS( xmm10, [xPointer + index * 16] )
					MOVAPS( xmm11, xmm10 )
					CMPEQPD( xmm10, xmm10 )
					PAND( xmm_temp, xmm10 )
					PANDN( xmm10, xmm11 )
					POR( xmm_temp, xmm10 )

				MAP_NAN( xmm0, 0 )
				MAP_NAN( xmm1, 1 )
				MAP_NAN( xmm2, 2 )
				MAP_NAN( xmm3, 3 )
				MAP_NAN( xmm4, 4 )
				MAP_NAN( xmm5, 5 )
				MAP_NAN( xmm6, 6 )
				MAP_NAN( xmm7, 7 )
				MAP_NAN( xmm8, 8 )
				MAP_NAN( xmm9, 9 )

				LOAD.CONSTANT( xmm15, one )

				JMP( 'horner' )

				return codegen.end_function()
